package com.example.demo;

import java.util.ArrayList;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.retry.annotation.EnableRetry;
import org.springframework.transaction.annotation.EnableTransactionManagement;

@SpringBootApplication
@EnableRetry
@EnableTransactionManagement
public class DemoApplication {
	private static final Logger LOGGER = LoggerFactory.getLogger(DemoApplication.class);

	public static void main(String[] args) {
		SpringApplication.run(DemoApplication.class, args);
	}

	@Bean
	CommandLineRunner runner(AService aService) {
		return args -> {
			aService.deleteAll();

			var id = "ac5cf47a-55a9-44a3-bb04-c2e88ff0a6b3";
			if (!aService.existsById(id)) {
				var newA = new A();
				newA.setId(id);
				aService.save(newA);
			}
			createBs(id, aService);
			aService.findById(id).ifPresent(a -> {
				if (a.getBs().size() != 10) {
					throw new RuntimeException("This should not happen");
				}
				System.out.println(a);
			});
		};
	}

	private void createBs(String aId, AService aService)
		throws InterruptedException, java.util.concurrent.ExecutionException {
		ExecutorService EXEC = Executors.newFixedThreadPool(10);
		var tasks = new ArrayList<Callable<String>>();
		for (int i = 0; i < 10; i++) {
			var c = new Callable<String>() {
				@Override
				public String call() {
					return aService.addB();
				}
			};
			tasks.add(c);
		}

		var results = EXEC.invokeAll(tasks);
		for (var fr : results) {
			System.out.println(fr.get());
		}
	}
}
